/* Copyright 1999-2004 Red Hat, Inc.
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <ctype.h>
#include <fcntl.h>
#include <ftw.h>
#include <limits.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>
#include <errno.h>

#include <sys/mount.h>
#include <sys/poll.h>
#include <sys/stat.h>
#include <sys/utsname.h>

#include <libintl.h>
#include <locale.h>

#include <newt.h>
#include <popt.h>

#define _(String) gettext((String))
#define N_(String) (String)

#include "kudzu.h"

#include "modules.h"

#define FILENAME "hwconf"

#define CARDSDB "/usr/share/hwdata/Cards"
#define XPATH "/usr/X11R6/bin/"
#define XMODPATH "/usr/X11R6/lib/modules/drivers/"

static int madebak=0;

struct device **storedDevs;
int numStored=0;
struct device **currentDevs;
int numCurrent=0;

//static int rhgb = 0;
static char *_module_file = NULL;
static char *kernel_ver = NULL;
int mouseconfigured = 0;

/* These need to be global, so we can kill them in case of problems */
pid_t wpid=0;
/*
void startNewt()
{
        char roottext[80];

        newtInit();
        newtCls();
        newtPushHelpLine(_(" <Tab>/<Alt-Tab> between elements   |   <Space> selects  |   <F12> next screen"));

        snprintf(roottext, 80,
                 _("Hardware Discovery Utility               "));
        newtDrawRootText(0, 0, roottext);
        if (!access("/usr/bin/rhgb-client",X_OK)) {
                system("/usr/bin/rhgb-client --details=yes >/dev/null 2>&1");
                rhgb = 1;
        }
}
*/

pid_t startwheel(void) /* Feedback while detecting hardware */
{
 char v[]="Autoconfiguring devices... ";
 char r[]="/-\\|";
 char *b="\b";
 pid_t pid;
 if((pid=fork())>0) return pid; /* return Child PID to Master process */
 else if(pid==0)
  {
   int i,j;
   /* Allow killing of process */
   signal(SIGHUP,SIG_DFL);
   signal(SIGINT,SIG_DFL);
   signal(SIGTERM,SIG_DFL);
   write(2,v,sizeof(v)-1);
   for(i=j=0;;i++,j++)
    {
     if(j%8==7) write(2,"\033[42;32m \033[0m",13); /* a green space */
     write(2,&r[i%4],1); write(2,b,1);
     usleep(40000);
    }
   exit(0); /* Exit child process */
  }
 return 0;
}

int exists(char *filename)
{
 struct stat s;
 return !stat(filename,&s);
}

struct xinfo {
char xserver[16];
char xmodule[16];
char xdesc[128];
char xopts[128];
};

struct xinfo *getxinfo(struct device *d)
{
 const char *xfree4="XFree86", *xvesa4="vesa";
 const char *xpath=XPATH;
 static struct xinfo xi;
 int rescanned=0;
 memset(&xi,0,sizeof(struct xinfo));
 if(d->desc) strncpy(xi.xdesc,d->desc,sizeof(xi.xdesc));
 if(d->driver)
  {
   const char *driver[]={"3DLabs",   "Mach64", "Mach32",    "Mach8",     "AGX",
                   "P9000", "S3 ViRGE",    "S3V",       "S3",     "W32",
                   "8514",     "I128",   "SVGA", xfree4, NULL};
   const char *server[]={driver[0], driver[1], driver[2], driver[3], driver[4],
                   driver[5], "S3V",     driver[7], driver[8], driver[9],
                   driver[10], driver[11],  driver[12], xfree4, NULL};
   if(!strncasecmp(d->driver,"Card:",5)) /* RedHat Cards-DB */
    { /* Kudzu "Cards" format */
     FILE *cardsdb;
     char xfree3server[128];
     memset(xfree3server,0,sizeof(xfree3server));
     if((cardsdb=fopen(CARDSDB,"r"))!=NULL)
      { /* Try to find Server and Module in /usr/share/kudzu/Cards */
       char buffer[1024];
       char searchfor[128];
       int found=0;
       memset(searchfor,0,sizeof(searchfor));
       sscanf(&d->driver[5],"%127[^\r\n]",searchfor);
       while(!found && !feof(cardsdb) && fgets(buffer,1024,cardsdb))
        {
         char sfound[128];
         memset(sfound,0,sizeof(sfound));
         if(strncasecmp(buffer,"NAME ",5) || 
           (sscanf(&buffer[5],"%127[^\r\n]",sfound)!=1) ||
           strcasecmp(sfound,searchfor)) continue;
         while(!feof(cardsdb) && fgets(buffer,1024,cardsdb))
          {
           if(buffer[0]<32) break; /* End-of-line */
           if(!strncasecmp(buffer,"SERVER ",7))
            {
             char x[20]="";
             if(sscanf(&buffer[7],"%19s",x)==1)
              {
               char xserver[32];
               char fullpath[128];
               char *xf[2]={"","XF86_"};
               int i;
               for(i=0;i<2;i++)
                {
                 sprintf(xserver,"%s%.24s",xf[i],x);
                 sprintf(fullpath,"%.90s%.32s",xpath,xserver);
                 if(exists(fullpath))
                  {
                   strncpy(xfree3server,xserver,sizeof(xfree3server));
                   break; /* for */
                  }
                }
              }
            }
           else if(!strncasecmp(buffer,"DRIVER ",7))
            {
             char xmodule[32];
             char fullpath[128];
             sscanf(&buffer[7],"%31s",xmodule);
             sprintf(fullpath,XMODPATH"%.31s_drv.o",xmodule);
             if(exists(fullpath))
              {
               strncpy(xi.xmodule,xmodule,sizeof(xi.xmodule));
              }
            }
           else if(!strncasecmp(buffer,"SEE ",4)&&rescanned<10)
            { /* rescan Cards-DB for other server */
             fseek(cardsdb,0L,SEEK_SET); ++rescanned;
             memset(searchfor,0,sizeof(searchfor));
             sscanf(&buffer[4],"%127[^\r\n]",searchfor);
             break; /* Continue with outer while() */
            }
          }
        }
       fclose(cardsdb);
      }
     if(*xi.xmodule||*xi.xserver||*xfree3server) /* (Partial) Success */
      {
       if(!*xi.xserver)
        {
         if(*xfree3server&&!*xi.xmodule)
           strncpy(xi.xserver,xfree3server,sizeof(xi.xserver));
         else
           strncpy(xi.xserver,xfree4,sizeof(xi.xserver));
        }
       if(!*xi.xmodule) strcpy(xi.xmodule,xvesa4);
       return &xi;
      }
    }
   /* Card not found in Cards database -> Try to guess from description */
   { int i;
   for(i=0;driver[i]!=NULL;i++)
    {
     if(strstr(d->driver,driver[i]))
      {
       char *xpos;
       if((xpos=strstr(d->driver,xfree4))!=NULL) /* Check for XFree 4 */
        {
         char xm[32]="";
         strcpy(xi.xserver,xfree4);
         if(sscanf(xpos,"XFree86(%30[^)])",xm)==1)
          strcpy(xi.xmodule,xm);
         else
          strcpy(xi.xmodule,xvesa4);
        }
       else
        {
         char xserver[32];
         char fullpath[128];
         char *xf[2]={"","XF86_"};
         int j;
         for(j=0;j<2;j++)
          {
           sprintf(xserver,"%s%.24s",xf[j],server[i]);
           sprintf(fullpath,"%.90s%.32s",xpath,xserver);
           if(exists(fullpath))
            {
             strncpy(xi.xserver,xserver,sizeof(xi.xserver));
             break; /* for */
            }
          }
        }
      }
    }
   }
  }
 /* Special options required? */
 if(d->desc)
  {
   strncpy(xi.xdesc,d->desc,sizeof(xi.xdesc)-1);
   /* Handle special cards that require special options */
   if(strstr(d->desc,"Trident")||strstr(d->desc,"TGUI")
          ||strstr(d->desc,"Cirrus")||strstr(d->desc,"clgd"))
    {
     if(!strcmp(xi.xserver,xfree4))
      strncpy(xi.xopts,"-depth 16",sizeof(xi.xopts)-1);
     else
      strncpy(xi.xopts,"-bpp 16",sizeof(xi.xopts)-1);
    }
   else if(strstr(d->desc,"Savage 4")) /* S3 Inc.|Savage 4 */
    {
     if(!strcmp(xi.xserver,xfree4))
      strncpy(xi.xopts,"-depth 32",sizeof(xi.xopts)-1);
     else
      strncpy(xi.xopts,"-bpp 32",sizeof(xi.xopts)-1);
    }
  }
 /* Fallback values */
 if(!*xi.xserver) strcpy(xi.xserver,xfree4);
 if(!*xi.xmodule) strcpy(xi.xmodule,xvesa4);
 return &xi;
}

int writeconfig(char *name,struct device *d)
{
 FILE *f,*k;
 const char *kconfig="/etc/sysconfig/BOOYO";
 unlink(name);
 if((f=fopen(name,"w"))==NULL)
  { /* Read-only filesystem on /etc ?! */
   fprintf(stderr,"Can't write to '%s': %s",name,strerror(errno));
   return 1;
  }
 if((k=fopen(kconfig,"a"))==NULL) { fclose(f); return 1; }
 switch(d->type)
  {
   case CLASS_AUDIO:
    {
     if(d->desc)   fprintf(f,"FULLNAME=\"%s\"\n",d->desc),
                   fprintf(k,"SOUND_FULLNAME=\"%s\"\n",d->desc);
     if(d->driver) fprintf(f,"DRIVER=\"%s\"\n",d->driver),
                   fprintf(k,"SOUND_DRIVER=\"%s\"\n",d->driver);
    }; break;
   case CLASS_MOUSE:
    {
     if(d->driver) fprintf(f,"DRIVER=\"%s\"\n",d->driver),
                   fprintf(k,"MOUSE_DRIVER=\"%s\"\n",d->driver); 
     if(d->desc)   fprintf(f,"FULLNAME=\"%s\"\n",d->desc),
                   fprintf(k,"MOUSE_FULLNAME=\"%s\"\n",d->desc);
     if(d->device) fprintf(f,"DEVICE=\"/dev/%s\"\n",d->device),
                   fprintf(k,"MOUSE_DEVICE=\"/dev/%s\"\n",d->device);
     mouseconfigured=1;
    }; break;
   case CLASS_NETWORK:
    {
     if(d->desc)   fprintf(f,"FULLNAME=\"%s\"\n",d->desc),
                   fprintf(k,"NETCARD_FULLNAME=\"%s\"\n",d->desc);
     if(d->driver) fprintf(f,"DRIVER=\"%s\"\n",d->driver),
                   fprintf(k,"NETCARD_DRIVER=\"%s\"\n",d->driver);
    }; break;
   case CLASS_VIDEO:
    {
     const char *xserver="XSERVER=\"%s\"\n";
     const char *xmodule="XMODULE=\"%s\"\n";
     const char *xopts="XOPTIONS=\"%s\"\n";
     const char *xdesc="XDESC=\"%s\"\n";
     struct xinfo *xi=getxinfo(d);
     if(xi)
      {
       if(*xi->xserver)
        { fprintf(f,xserver,xi->xserver); fprintf(k,xserver,xi->xserver); }
       if(*xi->xmodule)
        { fprintf(f,xmodule,xi->xmodule); fprintf(k,xmodule,xi->xmodule); }
       if(*xi->xopts)
        { fprintf(f,xopts,xi->xopts); fprintf(k,xopts,xi->xopts); }
       if(*xi->xdesc)
        { fprintf(f,xdesc,xi->xdesc); fprintf(k,xdesc,xi->xdesc); }
      }
    }; break;
   case CLASS_FLOPPY:
    {
     if(d->desc)   fprintf(f,"FULLNAME='%s'\n",d->desc),
                   fprintf(k,"FLOPPY_FULLNAME='%s'\n",d->desc);
     if(d->device) fprintf(f,"DEVICE=\"/dev/%s\"\n",d->device),
                   fprintf(k,"FLOPPY_DEVICE=\"/dev/%s\"\n",d->device);
     if(d->driver) fprintf(f,"DRIVER=\"%s\"\n",d->driver),
                   fprintf(k,"FLOPPY_DRIVER=\"%s\"\n",d->driver);
    }; break;
   default: break;
  }
 fclose(f); fclose(k);
 return 0;
}

int makeLink(struct device *dev, char *name)
{
	char oldfname[256],newfname[256];
	
	if (!dev->device || !name) return 1;
	snprintf(oldfname,256,"/dev/%s",dev->device);
	if (dev->index > 0) {
		snprintf(newfname,256,"/dev/%s%d",name,dev->index);
	} else {
		snprintf(newfname,256,"/dev/%s",name);
	}
	openlog("kudzu",0,LOG_USER);
	syslog(LOG_NOTICE,_("linked %s to %s"),newfname,oldfname);
	closelog();
	return symlink(oldfname,newfname);
}

int isLinked(struct device *dev, char *name)
{
	char path[256],path2[256];
	
	memset(path,'\0',256);
	memset(path2,'\0',256);
	if (!name) return 0;
	if (!dev->device) return 0;
	if (dev->index) 
	  snprintf(path,256,"/dev/%s%d",name,dev->index);
	else 
	  snprintf(path,256,"/dev/%s",name);
	if (readlink(path,path2,256)>0) {
		if (!strncmp(path2,"/dev/",5)) {
			if (!strcmp(path2+5,dev->device))
			  return 1;
		} else {
			if (!strcmp(path2,dev->device))
			  return 1;
		}
	}
	return 0;
}

static inline int isSerialish(struct device *dev) {
	return (dev->device && (
			    !strcmp(dev->device, "console") || 
			    !strncmp(dev->device, "ttyS", 4) || 
			    !strcmp(dev->device, "hvc0") ||
			    !strcmp(dev->device, "ttySG0") ||
			    !strncmp(dev->device, "hvsi", 4) ));
}

/* HACK. This is so not thread-safe. */
static char mod_name[100], cmod_name[100]; 

static int isModule(const char *filename, const struct stat *sb, int flag) {
	char *fname = basename(filename);
	/* ugly hack; ignore things under build symlink */
	if ((!strcmp(fname,mod_name) || !strcmp(fname,cmod_name)) && !strstr(filename,"/build/")) {
		return 1;
	}
	return 0;
}


int isAvailable(char *modulename)
{
	struct stat sbuf;
	char path[512];
	int x;
	char *searchdir[] = {
		"fs",
		"net",
		"scsi",
		"block",
		"cdrom",
		"ipv4",
		"ipv6",
		"sound",
		"fc4",
		"video",
		"misc",
		"pcmcia",
		"atm",
		"usb",
		NULL
	};
	
	snprintf(mod_name,100,"%s.ko",modulename);
	snprintf(cmod_name,100,"%s.ko.gz",modulename);
	snprintf(path,512,"/lib/modules/%s",kernel_ver);
	/* Do not set the third argument of this function to < 6. Blarg. */
	if (ftw(path,isModule,15) == 1) {
		return 1;
	}
	/* Finally, try non-version specific directories. */
	for (x = 0; searchdir[x]; x++) {
		snprintf(path,512,"/lib/modules/%s/%s.o",
			 searchdir[x],modulename);
		if (!stat(path,&sbuf))
		  return 1;
		snprintf(path,512,"/lib/modules/%s/%s.o.gz",
			 searchdir[x],modulename);
		if (!stat(path,&sbuf))
		  return 1;
		snprintf(path,512,"/lib/modules/%s/%s.ko",
			 searchdir[x],modulename);
		if (!stat(path,&sbuf))
		  return 1;
	}
	return 0;
}

int isConfigurable(struct device *dev) {
	struct stat;
	
	if (!strcmp(dev->driver,"parport_serial"))
		return 1;
	switch (dev->type) {
	 case CLASS_NETWORK:
	 case CLASS_SCSI:
	 case CLASS_IDE:
	 case CLASS_RAID:
	 case CLASS_CAPTURE:
	 case CLASS_USB:
	 case CLASS_FIREWIRE:
	 case CLASS_KEYBOARD:
	 case CLASS_MODEM:
	 case CLASS_SCANNER:
	 case CLASS_PRINTER:
		return 1;
	 case CLASS_MOUSE:
		return 1;
	 case CLASS_AUDIO:
		return 1;
	 case CLASS_FLOPPY:
		return 1;
	 case CLASS_VIDEO:
		return 1;
	 case CLASS_OTHER:
		if (dev->bus == BUS_PCI && 
		    ((struct pciDevice *)dev)->vendorId == 0x14e4 &&
		    ((struct pciDevice *)dev)->deviceId == 0x5820)
			return 1;
		return 0;
	 case CLASS_TAPE:
	 case CLASS_HD:
	 case CLASS_CDROM:
	 case CLASS_SOCKET:
	 default:
		return 0;
	}
	return 0;
}

int isConfigured(struct device *dev)
{
	struct confModules *cf;
	char path[256], path2[256];
	struct stat sbuf;
	int ret=0;
	
	memset(path,'\0',256);
	memset(path2,'\0',256);
	cf = readConfModules(_module_file);
	switch (dev->type) {
	 case CLASS_NETWORK:
		if (!strcmp(dev->driver,"unknown") ||
		    !strcmp(dev->driver,"ignore") ||
		    !strcmp(dev->driver,"disabled"))
		  ret=1;
		/* Assume cardbus stuff is unconfigured */
		if (dev->bus == BUS_PCMCIA)
			break;
		if (dev->bus != BUS_PCI || ((struct pciDevice *)dev)->pciType != PCI_CARDBUS)
		  if (cf)
		    if (dev->device && isAliased(cf,dev->device,dev->driver)!=-1)
		      ret = 1;
		break;
	 case CLASS_RAID:
	 case CLASS_SCSI:
	 case CLASS_IDE:
		if (!strcmp(dev->driver,"unknown") ||
		    !strcmp(dev->driver,"disabled") ||
		    !strcmp(dev->driver,"ignore"))
		  ret=1;
		if (cf)
		  if (isAliased(cf,"scsi_hostadapter",dev->driver)!=-1)
		    ret=1;
		break;
	 case CLASS_VIDEO:
	 case CLASS_MONITOR:
		/* Assume on initial runs that if X is configured, we got the right card */
		//ret = 1;
		if (!strcmp(dev->driver,"unknown") ||
		    !strcmp(dev->driver,"disabled") ||
		    !strcmp(dev->driver,"ignore"))
		  ret=1;
		break;
	 case CLASS_CAPTURE:
		if (!strcmp(dev->driver,"unknown") ||
		    !strcmp(dev->driver,"disabled") ||
		    !strcmp(dev->driver,"ignore"))
		  ret=1;
		if (cf)
		  if (isAliased(cf,"char-major-81",dev->driver)!=-1)
		    ret = 1;
		break;
	 case CLASS_AUDIO:
		if (!strcmp(dev->driver,"unknown") ||
		    !strcmp(dev->driver,"disabled") ||
		    !strcmp(dev->driver,"ignore"))
		  ret=1;
		if (cf) {
			if (isAliased(cf,"sound",dev->driver)!=-1)
			  ret = 1;
			if (isAliased(cf,"sound-card-",dev->driver)!=-1)
			  ret = 1;
			if (isAliased(cf,"sound-slot-",dev->driver)!=-1)
			  ret = 1;
			if (isAliased(cf,"snd-card-",dev->driver)!=-1)
			  ret = 1;
		}
		break;
	 case CLASS_USB:
	        if (!strcmp(dev->driver,"unknown") ||
		    !strcmp(dev->driver,"disabled") ||
		    !strcmp(dev->driver,"ignore"))
		    ret = 1;
		if (cf && isAliased(cf,"usb-controller",dev->driver)!=-1)
		  ret = 1;
		break;
	 case CLASS_FIREWIRE:
	        if (!strcmp(dev->driver,"unknown") ||
		    !strcmp(dev->driver,"disabled") ||
		    !strcmp(dev->driver,"ignore"))
		    ret = 1;
		if (cf && isAliased(cf,"ieee1394-controller",dev->driver)!=-1)
		  ret = 1;
		break;
	 case CLASS_MOUSE:
		{
			int fd;
			char *buf, *tmp, devpath[256];

			fd = open("/etc/sysconfig/mouse",O_RDONLY);
			if (fd < 0) {
				ret = 0;
				break;
			}
			buf = bufFromFd(fd);
			tmp = strstr(buf,"DEVICE=");
			if (!tmp) {
				ret = 0;
				break;
			}
			if (tmp == buf || *(tmp-1) == '\n') {
				snprintf(devpath,255,"/dev/%s",dev->device);
				if (!strncmp(tmp+7,devpath, strlen(devpath))) {
					ret = 1;
					break;
				}
			}
			ret = 0;
		}
		break;
			
	 case CLASS_MODEM:
		ret = isLinked(dev,"modem");
		break;
	 case CLASS_SCANNER:
		ret = isLinked(dev,"scanner");
		break;
	 case CLASS_KEYBOARD:
		break;	 
	 case CLASS_PRINTER:
		/* fairly braindead printcap parser */
		{
			char *buf,*ptr,*tmp;
			int fd;
			
			fd = open("/etc/printcap",O_RDONLY);
			if (fd==-1) break;
			fstat(fd,&sbuf);
			buf=malloc(sbuf.st_size+1);
			if (read(fd,buf,sbuf.st_size)!=sbuf.st_size) break;
			buf[sbuf.st_size] = '\0';
			ptr=buf;
			while (buf[0]!='\0') {
				if (ptr[0] == '#') {
					while (*ptr && *ptr != '\n') ptr++;
					if (*ptr) {
						*ptr='\0';
						ptr++;
					}
					buf=ptr;
		                        continue;
				}
				while (*ptr && *ptr != '\0' && *ptr !='\n') ptr++;
				if (*ptr) {
					*ptr='\0';
					ptr++;
				}
				if ((tmp=strstr(buf,"lp=/dev/"))) {
					while (*tmp && *tmp != ':') tmp++;
					if (*tmp) {
						*tmp = '\0';
					}
					if (!strcmp(buf+8,dev->device))
					  ret = 1;
				}
				buf = ptr;
			}
			break;
		}
	 case CLASS_OTHER:
		if (dev->bus == BUS_PCI && 
		    ((struct pciDevice *)dev)->vendorId == 0x14e4 &&
		    ((struct pciDevice *)dev)->deviceId == 0x5820)
			return 0;
		return 1;
	 default:
		/* If we don't know how to configure it, assume it's configured. */
		ret = 1;
		break;
	}
	if (cf)
	  freeConfModules(cf);
	return ret;
}

int configure(struct device *dev)
{
	struct confModules *cf;
	char path[256];
	int index;
	static int current_netdev = 0;

	if (!strcmp(dev->driver,"parport_serial")) {
		cf = readConfModules(_module_file);
		if (!cf)
		  cf = newConfModules();
		cf->madebackup = madebak;
		addAlias(cf,"parport_lowlevel","parport_serial", CM_REPLACE);
		writeConfModules(cf,_module_file);
		madebak = cf->madebackup;
		openlog("kudzu",0,LOG_USER);
		syslog(LOG_NOTICE,_("aliased %s as %s"),"parport_lowlevel","parport_serial");
		closelog();
	} else switch (dev->type) {
	 case CLASS_NETWORK:
		if (!dev->device) break;
		cf = readConfModules(_module_file);
		if (!cf) 
		  cf = newConfModules();
		cf->madebackup = madebak;
		if (isAliased(cf,dev->device,dev->driver)) {
			if ( (dev->bus != BUS_PCI && dev->bus != BUS_PCMCIA) || 
			     ((struct pciDevice *)dev)->pciType != PCI_CARDBUS) {
				addAlias(cf,dev->device,dev->driver,CM_REPLACE);
				writeConfModules(cf,_module_file);
				madebak = cf->madebackup;
				openlog("kudzu",0,LOG_USER);
				syslog(LOG_NOTICE,_("aliased %s as %s"),path,dev->driver);
				closelog();
			}
		}
		freeConfModules(cf);
		current_netdev++;
		writeconfig("/etc/sysconfig/netcard",dev);
		break;
	 case CLASS_RAID:
	 case CLASS_IDE:
	 case CLASS_SCSI:
		cf = readConfModules(_module_file);
		if (!cf)
		  cf = newConfModules();
		cf->madebackup = madebak;
		if (isAliased(cf,"scsi_hostadapter",dev->driver)==-1) {
			index=0;
			while (1) {
				if (index)
				  snprintf(path,256,"scsi_hostadapter%d",index);
				else
				  snprintf(path,256,"scsi_hostadapter");
				if (getAlias(cf,path)) 
				  index++;
				else
				  break;
			}
			addAlias(cf,path,dev->driver,CM_REPLACE);
			writeConfModules(cf,_module_file);
			madebak = cf->madebackup;
			openlog("kudzu",0,LOG_USER);
			syslog(LOG_NOTICE,_("aliased %s as %s"),path,dev->driver);
			closelog();
		}
		freeConfModules(cf);
		break;
	 case CLASS_VIDEO:
		//cmk
		//configuredX = Xconfig(dev);
		writeconfig("/etc/sysconfig/xserver",dev);
		break;
	 case CLASS_CAPTURE:
		cf = readConfModules(_module_file);
		if (!cf)
		  cf = newConfModules();
		cf->madebackup = madebak;
		if (isAliased(cf,"char-major-81",dev->driver)==-1) {
			snprintf(path,256,"char-major-81");
			addAlias(cf,path,dev->driver,CM_REPLACE);
			writeConfModules(cf,_module_file);
			madebak = cf->madebackup;
			openlog("kudzu",0,LOG_USER);
			syslog(LOG_NOTICE,_("aliased %s as %s"),path,dev->driver);
			closelog();
		}
		freeConfModules(cf);
		break;
	 case CLASS_USB:
		cf = readConfModules(_module_file);
		if (!cf)
		  cf = newConfModules();
		cf->madebackup = madebak;
		if (isAliased(cf,"usb-controller",dev->driver)==-1) {
			index=0;
			while (1) {
				if (index)
				  snprintf(path,256,"usb-controller%d",index);
				else
				  snprintf(path,256,"usb-controller");
				if (getAlias(cf,path))
				  index++;
				else
				  break;
			}
			addAlias(cf,path,dev->driver,CM_REPLACE);
			writeConfModules(cf,_module_file);
			madebak = cf->madebackup;
			openlog("kudzu",0,LOG_USER);
			syslog(LOG_NOTICE,_("aliased %s as %s"),path,dev->driver);
			closelog();
		}
		freeConfModules(cf);
		break;
	 case CLASS_FIREWIRE:
		cf = readConfModules(_module_file);
		if (!cf)
		  cf = newConfModules();
		cf->madebackup = madebak;
		if (isAliased(cf,"ieee1394-controller",dev->driver)==-1) {
			index=0;
			while (1) {
				if (index)
				  snprintf(path,256,"ieee1394-controller%d",index);
				else
				  snprintf(path,256,"ieee1394-controller");
				if (getAlias(cf,path))
				  index++;
				else
				  break;
			}
			addAlias(cf,path,dev->driver,CM_REPLACE);
			writeConfModules(cf,_module_file);
			madebak = cf->madebackup;
			openlog("kudzu",0,LOG_USER);
			syslog(LOG_NOTICE,_("aliased %s as %s"),path,dev->driver);
			closelog();
		}
		freeConfModules(cf);
		break;
	 case CLASS_AUDIO:
		cf = readConfModules(_module_file);
		if (!cf)
			cf = newConfModules();
		cf->madebackup = madebak;
		if (
		    isAliased(cf,"sound-slot-",dev->driver)==-1 &&
		    isAliased(cf,"sound-card-",dev->driver)==-1 &&
		    isAliased(cf,"snd-card-",dev->driver)==-1
		    ) {
			snprintf(path,256,"snd-card-%d",dev->index);
			addAlias(cf,path,dev->driver,CM_REPLACE);
			snprintf(path,256,"options snd-card-%d index=%d",dev->index,dev->index);
			addLine(cf,path,CM_REPLACE);
			snprintf(path,256,"install %s /sbin/modprobe --ignore-install %s && /usr/sbin/alsactl restore >/dev/null 2>&1 || :", dev->driver, dev->driver);
			addLine(cf,path,CM_REPLACE);
			snprintf(path,256,"remove %s { /usr/sbin/alsactl store >/dev/null 2>&1 || : ; }; /sbin/modprobe -r --ignore-remove %s",dev->driver, dev->driver);
			addLine(cf,path,CM_REPLACE);
			writeConfModules(cf,_module_file);
			madebak = cf->madebackup;
			openlog("kudzu",0,LOG_USER);
			snprintf(path,256,"snd-card-%d",dev->index);
			syslog(LOG_NOTICE,_("aliased %s as %s"),path,dev->driver);
			closelog();
		}
		writeconfig("/etc/sysconfig/sound",dev);
		break;
	 case CLASS_MOUSE:
		if (!strcmp(dev->driver,"ignore")) break;
                writeconfig("/etc/sysconfig/mouse",dev);
                makeLink(dev,"mouse");
		break;
	 case CLASS_KEYBOARD:
		break;
	 case CLASS_MODEM:
		makeLink(dev,"modem");
		break;
	 case CLASS_SCANNER:
		makeLink(dev,"scanner");
		break;
	 case CLASS_PRINTER:
		//newtSuspend();
		if (dev->bus == BUS_PARALLEL) {
			snprintf(path,256,"/usr/sbin/printconf-tui --Xadd-local --device=/dev/%s --make=\"%s\" --model=\"%s\" >/dev/null 2>&1",
				 dev->device,
				 ((struct parallelDevice *)dev)->pnpmfr,
				 ((struct parallelDevice *)dev)->pnpmodel);
		} else if (dev->bus == BUS_USB) {
			if (((struct usbDevice *)dev)->usbmfr &&
			    ((struct usbDevice *)dev)->usbprod)
				snprintf(path,256,"/usr/sbin/printconf-tui --Xadd-local --device=/dev/usb/lp0 --make=\"%s\" --model=\"%s\" >/dev/null 2>&1",
				 ((struct usbDevice *)dev)->usbmfr,
				 ((struct usbDevice *)dev)->usbprod);
			else break;
		} else
			break;
                system(path);
                //newtResume();
		openlog("kudzu",0,LOG_USER);
		syslog(LOG_NOTICE,_("ran printconf for %s"),dev->desc);
		closelog();
		break;
	 case CLASS_OTHER:
		if (dev->bus == BUS_PCI && 
		    !strcmp(dev->driver,"bcm5820")) {
			int retcode;
			
			retcode = system("/sbin/chkconfig --level 345 bcm5820 on > /dev/null 2>&1");
			if (retcode == 0) {
				openlog("kudzu",0,LOG_USER);
				syslog(LOG_NOTICE,_("turned on bcm5820 service"));
				closelog();
			}
		}
		if (dev->bus == BUS_PCI && 
		    !strcmp(dev->driver,"paep")) {
			int retcode;
			
			retcode = system("/sbin/chkconfig --level 345 aep1000 on > /dev/null 2>&1");
			if (retcode == 0) {
				openlog("kudzu",0,LOG_USER);
				syslog(LOG_NOTICE,_("turned on aep1000 service"));
				closelog();
			}
		}
		break;
	 case CLASS_TAPE:
	 case CLASS_FLOPPY:
	 case CLASS_HD:
	 default:
		break;
	}
	return 0;
}

void configMenu(struct device *newDevs)
{
	struct device *dev;

	dev = newDevs;
	for ( ; dev; dev=dev->next) {
		if (isConfigurable(dev) &&
		    !(dev->bus == BUS_PCI &&  dev->type != CLASS_MODEM &&
		      (!strcmp(dev->driver, "ignore") || 
		       !strcmp(dev->driver, "disabled") ||
		       !strcmp(dev->driver, "unknown"))) ) {
		  if (!isConfigured(dev)) {
			  switch (dev->type) {
			   case CLASS_NETWORK:
			   case CLASS_SCSI:
			   case CLASS_IDE:
			   case CLASS_RAID:
			   case CLASS_CAPTURE:
			   case CLASS_AUDIO:
			   case CLASS_USB:
			   case CLASS_FIREWIRE:
			   case CLASS_OTHER:
				  if (isAvailable(dev->driver))
				    continue;
				  break;
			   /* If we are running for the first time, and they
			    * have a mouse that currently exists configured,
			    * ignore any secondary mice. */
			   case CLASS_MOUSE:
				  if (mouseconfigured)
				    break;
				  continue;
			   default:
				  continue;
			  }

		  }
		}
		newDevs = listRemove(newDevs, dev);
	}
	if (!newDevs)
	  return;

	//startNewt();
	
	dev = newDevs;
	for ( ; dev ; dev = dev->next)
		configure(dev);
}

static void setupKernelVersion() {
        if (!kernel_ver) {
                struct utsname utsbuf;

                uname(&utsbuf);
                kernel_ver = strdup(utsbuf.release);
        }
}

int main(int argc, char **argv) 
{
	int x;
	enum deviceBus probeBus = BUS_UNSPEC;
        enum deviceClass probeClass = CLASS_UNSPEC;

        setlocale(LC_ALL, "");
        bindtextdomain("hwsetup-booyo", "/usr/share/locale");
        textdomain("hwsetup-booyo");
	
	if (getuid()) {
		fprintf(stderr,
			_("\nERROR - You must be root to run kudzu.\n"));
		exit(1);
	}

	setupKernelVersion();
	_module_file = "/etc/modprobe.conf";
	
	initializeBusDeviceList(probeBus);
	currentDevs = probeDevices(probeClass, probeBus, PROBE_ALL);
	writeDevices("/etc/sysconfig/hwconf",currentDevs);
	if (!currentDevs) {
		exit(0);
	}
	while (currentDevs[numCurrent]) numCurrent++;
	freeDeviceList();
	currentDevs[0]->next = NULL;
	for (x=1;currentDevs[x];x++) 
		currentDevs[x-1]->next = currentDevs[x];
	currentDevs[x-1]->next = NULL;
	configMenu((*currentDevs));
	return 0;
}
